package com.utils.open.wechat;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;

import android.content.Context;
import android.util.Log;
import android.widget.Toast;

import com.tencent.mm.sdk.modelmsg.SendAuth;
import com.tencent.mm.sdk.modelpay.PayReq;
import com.tencent.mm.sdk.openapi.IWXAPI;
import com.tencent.mm.sdk.openapi.WXAPIFactory;
import com.utils.open.wechat.common.MD5;
import com.utils.open.wechat.common.Util;

public class OpenWeChatInterface  {

	private static final String DEBUG_TAG = "open wechat";
	//涓婁笅鏂囩幆澧�
	private Context mContext;
    //绉佹湁鐨勯粯璁ゆ瀯閫犲瓙  
    private OpenWeChatInterface() {} 
    //宸茬粡鑷瀹炰緥鍖�  
    private static final OpenWeChatInterface instance = new OpenWeChatInterface();  
    //寰俊鍙傛暟
    private String APP_ID,APP_SECRET,MCH_ID,API_KEY;
    //闈欐�宸ュ巶鏂规硶   
    public static OpenWeChatInterface getInstance() 
    {  
        return instance;  
    }
    //API瀹炰緥
    private IWXAPI api ;
    
    public static void initSDK(Context context) {
		//微信初始化
    	try
    	{
    		OpenWeChatInterface.getInstance().init(context);
    	}
    	catch(Exception e)
    	{
    		e.printStackTrace();
    	}
    }
    
    public static void paySDK(String prepayID) {
		try
		{
			OpenWeChatInterface.getInstance().pay(prepayID);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
    }
    
    //娉ㄥ唽
    public void init(Context context)
    {
    	mContext = context;
    	
    	APP_ID = (String)Util.getMetaData(context, "WX_APPID");
    	APP_SECRET = (String)Util.getMetaData(context, "WX_APPSECRET");
    	MCH_ID = String.valueOf(Util.getMetaData(context, "WX_MCHID"));
    	API_KEY = (String)Util.getMetaData(context, "WX_APIKEY");
    	
    	WeiChatConstants.setAPI_KEY(API_KEY);
    	WeiChatConstants.setAPP_ID(APP_ID);
    	WeiChatConstants.setAPP_SECRET(APP_SECRET);
    	WeiChatConstants.setMCH_ID(MCH_ID);

    	api = WXAPIFactory.createWXAPI(context, null);
		// 灏嗚app娉ㄥ唽鍒板井淇�
		api.registerApp(APP_ID);
    }    
    
    //鎺堟潈鐧诲綍
    public void auth()
    {
    	if(!api.isWXAppInstalled())
    	{
    		Toast.makeText(mContext, "你没有安装微信，请先安装微信", Toast.LENGTH_SHORT).show();
    		return;
    	}
		// send oauth request 
		final SendAuth.Req req = new SendAuth.Req();
		req.scope = "snsapi_userinfo";
		req.state = "wechat_auth";
		api.sendReq(req);
    }

    //璇锋眰AccessToken
    public String getAccessToken(String code)
    {
    	String urlStr = "https://api.weixin.qq.com/sns/oauth2/access_token?" +
    			"appid="+APP_ID +
    			"&secret="+APP_SECRET +
    			"&code="+code + 
    			"&grant_type=authorization_code";
    	
    	String reqResultStr = executeHttpGet(urlStr);
    	Log.e("Wechat", "reqResult = " + reqResultStr);
    	if(reqResultStr == null)
    	{
    		return null;
    	}
    	
    	return reqResultStr;
    }
    
    //鑾峰彇涓汉璧勬枡
    public String getUserInfo(String accessToken,String openID)
    {
    	String urlStr = "https://api.weixin.qq.com/sns/userinfo?access_token="+accessToken +
    			"&openid="+openID; 
    	
    	String reqResultStr = executeHttpGet(urlStr);
    	Log.e("Wechat", "reqResult = " + reqResultStr);
    	if(reqResultStr == null)
    	{
    		return null;
    	}
    	
    	return reqResultStr;
    }
   
    public String executeHttpGet(String urlStr) {
    	
        String result = null;
        URL url = null;
        HttpURLConnection connection = null;
        InputStreamReader in = null;
        try {
            url = new URL(urlStr);
            connection = (HttpURLConnection) url.openConnection();
            in = new InputStreamReader(connection.getInputStream());
            BufferedReader bufferedReader = new BufferedReader(in);
            StringBuffer strBuffer = new StringBuffer();
            String line = null;
            while ((line = bufferedReader.readLine()) != null) {
                strBuffer.append(line);
            }
            result = strBuffer.toString();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (connection != null) {
                connection.disconnect();
            }
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
 
        }
        return result;
    }
    
    //鏀粯
    public void pay(String prepayID)
    {
    	if(!api.isWXAppInstalled())
    	{
    		Toast.makeText(mContext, "你没有安装微信，请先安装微信", Toast.LENGTH_SHORT).show();
    		return;
    	}
    	
    	Log.e("wechat pay", "prepayID = " + prepayID);
    	PayReq request = new PayReq();
    	request.appId = APP_ID;
    	request.partnerId = MCH_ID;
    	request.prepayId= prepayID;
    	request.packageValue = "Sign=WXPay";
    	request.nonceStr= genNonceStr();
    	request.timeStamp= String.valueOf(genTimeStamp());

    	List<NameValuePair> signParams = new LinkedList<NameValuePair>();
		signParams.add(new BasicNameValuePair("appid", request.appId));
		signParams.add(new BasicNameValuePair("noncestr", request.nonceStr));
		signParams.add(new BasicNameValuePair("package", request.packageValue));
		signParams.add(new BasicNameValuePair("partnerid", request.partnerId));
		signParams.add(new BasicNameValuePair("prepayid", request.prepayId));
		signParams.add(new BasicNameValuePair("timestamp", request.timeStamp));

		request.sign = genAppSign(signParams);
    	
    	api.sendReq(request);
    }
    
	private String genAppSign(List<NameValuePair> params) {
		StringBuilder sb = new StringBuilder();

		for (int i = 0; i < params.size(); i++) {
			sb.append(params.get(i).getName());
			sb.append('=');
			sb.append(params.get(i).getValue());
			sb.append('&');
		}
		sb.append("key=");
		sb.append(API_KEY);

		String appSign = MD5.getMessageDigest(sb.toString().getBytes()).toUpperCase();
		return appSign;
	}
    
    private String genNonceStr() {
		Random random = new Random();
		return MD5.getMessageDigest(String.valueOf(random.nextInt(10000)).getBytes());
	}
	
	private long genTimeStamp() {
		return System.currentTimeMillis() / 1000;
	}
	
}
