--region *.lua
--Date
--JNI通过C++调用到lua的接口
local NetMgr = require("net/NetMgr")
local JNI_EventID = {}
JNI_EventID.XZDD_JNI_NETWORKSTATECHANGED = "XZDD_JNI_NetworkStateChanged"    -- 网络变化
JNI_EventID.XZDD_JNI_DUMPENERGYCHANGED = "XZDD_JNI_DumpEnergyChanged"    -- 电量变化
JNI_EventID.XZDD_JNI_USERCENTERHEADPHOTO = "XZDD_JNI_UserCenterHeadPhoto"    -- 更换头像通知
JNI_EventID.XZDD_JNI_UPDATEALUMPHOTO = "XZDD_JNI_updateAlbumPhoto"    -- 相册上传通知
JNI_EventID.XZDD_JNI_GOGAMEPLAYS = "XZDD_JNI_GoGamePlays"    -- 去某个玩法页
JNI_EventID.XZDD_JNI_STARTGAME = "XZDD_JNI_startGame"    -- 开始游戏
JNI_EventID.XZDD_JNI_ONJSCALL = "XZDD_JNI_OnjsCall"    -- 购买或者推荐
JNI_EventID.XZDD_JNI_ENTERBACKGROUND = "XZDD_JNI_ENTERBACKGROUND"  -- 进入后台
JNI_EventID.XZDD_JNI_ENTERFOREGROUND = "XZDD_JNI_ENTERFOREGROUND"  -- 进入前台
JNI_EventID.kEventDDZGameStart = "kEventDDZGameStart"  -- 开始游戏
JNI_EventID.kEventDDZGameEnd = "kEventDDZGameEnd"  -- 游戏结束


JNI_EventID.OBaseGameScene_OnEnter = "OBaseGameScene_OnEnter"  -- 进入房间事件
JNI_EventID.OBaseGameScene_OnExit = "OBaseGameScene_OnExit"  -- 房间退出事件

local Jni_dispatchCustomEvent = function (eventName, ...)
    -- body
    assert(type(eventName) == "string", debug.traceback())
    local event = cc.EventCustom:new(eventName)
    event._userdata = { ...}
    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
    if #event._userdata >1 then--如果转发给c++的事件，必须在第二个参数里面带上类型描述串
        NotificationCenterPost(eventName,event._userdata[1],event._userdata[2])
    else
        NotificationCenterPost(eventName)
    end
end

local Jni_dispatchCustomEventFromC = function (eventName, ...)
    -- body
    assert(type(eventName) == "string", debug.traceback())
    local event = cc.EventCustom:new(eventName)
    event._userdata = { ...}
    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
end


--网络变化
function JNIBridgeLua_onNetworkStateChanged(value)
    Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_NETWORKSTATECHANGED , value )
end 

--电量变化
function JNIBridgeLua_onDumpEnergyChanged(value)
    Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_DUMPENERGYCHANGED , value )
end

--头像更换通知
function JNIBridgeLua_onUserCenterHeadPhoto()
    Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_USERCENTERHEADPHOTO )
end

--上传相册图片通知
function JNIBridgeLua_updateAlbumPhoto()
    Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_UPDATEALUMPHOTO )

end

--去某个玩法页
function JNIBridgeLua_onGoGamePlays(value)
    Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_GOGAMEPLAYS, value )
end

--开始游戏
function JNIBridgeLua_startGame(value)
    Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_STARTGAME, value )
end

--购买商品，推荐情景
function JNIBridgeLua_OnjsCall(args)
    --args表示两个参数组合的字符串
    local Result = GlobalFunction.SplitStr(args,"&");
    if Result.length ~= nil and Result.length > 1 then
        Jni_dispatchCustomEvent( JNI_EventID.XZDD_JNI_ONJSCALL, tonumber(Result[1]),tonumber(Result[1]))
    else
        print("JNIBridgeLua_OnjsCall 参数不够两个")
    end
end

-- 进入后台事件
function JNIBridgeLua_onEnterBackground()
    print("JNIBridgeLua_onEnterBackground")
    Jni_dispatchCustomEvent(JNI_EventID.XZDD_JNI_ENTERBACKGROUND)
end

-- 进入前台事件
function JNIBridgeLua_onEnterForeground()
    print("JNIBridgeLua_onEnterForeground")
    Jni_dispatchCustomEvent(JNI_EventID.XZDD_JNI_ENTERFOREGROUND)
end 

-- 房间进入事件
function JNIBridgeLua_OBaseGameScene_OnEnter()
    print("JNIBridgeLua_OBaseGameScene_OnEnter")
    NetMgr.onMessage(JNI_EventID.OBaseGameScene_OnEnter)
end 

-- 房间退出事件
function JNIBridgeLua_OBaseGameScene_OnExit()
    print("JNIBridgeLua_OBaseGameScene_OnExit")
    NetMgr.onMessage(JNI_EventID.OBaseGameScene_OnExit)
end 


-- 游戏开始事件
function JNIBridgeLua_OBaseGameScene_DDZGameStart()
    print("JNIBridgeLua_OBaseGameScene_DDZGameStart")
    NetMgr.onMessage(JNI_EventID.kEventDDZGameStart)
end 

-- 游戏结束事件
function JNIBridgeLua_OBaseGameScene_DDZGameEnd()
    print("JNIBridgeLua_OBaseGameScene_DDZGameEnd")
    NetMgr.onMessage(JNI_EventID.kEventDDZGameEnd)
end 



--C++__NotificationCenter::postNotification接口传过来的事件
function NotificationCenter_postNotification(eventName,param)
    Jni_dispatchCustomEventFromC(eventName,param)
end



--endregion
